<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en-us">
<head>
   <meta http-equiv="content-type" content="text/html; charset=utf-8" />
   <title></title>
   <meta name="author" content="<%= @page.author %>" />

   <!-- CodeRay syntax highlighting CSS -->
   <link rel="stylesheet" href="/css/coderay.css" type="text/css" />

   <!-- Homepage CSS -->
   <link rel="stylesheet" href="/css/screen.css" type="text/css" media="screen, projection" />
</head>
<body>

<div class="site">
  <div class="title">
    Tom Preston-Werner
  </div>

  <p>under default configuration, this post should get processed by the identity converter. By changing 
textile extension or markdown extension configuration parameters, you should be able to associate
it with either of those converters</p>

</div>

</body>
</html>
